@@ -171,7 +171,7 @@ class AbstractModel {
                         const AmiVector &x, const AmiVector &dx) = 0;
 
     /**
-     * @brief Model specific sparse implementation of explicit parameter
+     * @brief Model-specific sparse implementation of explicit parameter
      * derivative of right hand side
      * @param t time
      * @param x state
@@ -195,19 +195,19 @@ class AbstractModel {
                      realtype cj) = 0;
 
     /**
-     * @brief Returns the amici version that was used to generate the model
-     * @return ver amici version string
+     * @brief Returns the AMICI version that was used to generate the model
+     * @return AMICI version string
      */
-    virtual const std::string getAmiciVersion() const;
+    virtual std::string getAmiciVersion() const;
 
     /**
-     * @brief Returns the amici commit that was used to generate the model
-     * @return ver amici commit string
+     * @brief Returns the AMICI commit that was used to generate the model
+     * @return AMICI commit string
      */
-    virtual const std::string getAmiciCommit() const;
+    virtual std::string getAmiciCommit() const;
 
     /**
-     * @brief Model specific implementation of fx0
+     * @brief Model-specific implementation of fx0
      * @param x0 initial state
      * @param t initial time
      * @param p parameter vector
@@ -225,7 +225,7 @@ class AbstractModel {
     virtual bool isFixedParameterStateReinitializationAllowed() const;
 
     /**
-     * @brief Model specific implementation of fx0_fixedParameters
+     * @brief Model-specific implementation of fx0_fixedParameters
      * @param x0 initial state
      * @param t initial time
      * @param p parameter vector
@@ -238,7 +238,7 @@ class AbstractModel {
                                      gsl::span<const int> reinitialization_state_idxs);
 
     /**
-     * @brief Model specific implementation of fsx0_fixedParameters
+     * @brief Model-specific implementation of fsx0_fixedParameters
      * @param sx0 initial state sensitivities
      * @param t initial time
      * @param x0 initial state
@@ -254,7 +254,7 @@ class AbstractModel {
                                       gsl::span<const int> reinitialization_state_idxs);
 
     /**
-     * @brief Model specific implementation of fsx0
+     * @brief Model-specific implementation of fsx0
      * @param sx0 initial state sensitivities
      * @param t initial time
      * @param x0 initial state
@@ -275,7 +275,7 @@ class AbstractModel {
     virtual void fdx0(AmiVector &x0, AmiVector &dx0);
 
     /**
-     * @brief Model specific implementation of fstau
+     * @brief Model-specific implementation of fstau
      * @param stau total derivative of event timepoint
      * @param t current time
      * @param x current state
@@ -291,7 +291,7 @@ class AbstractModel {
                        const realtype *sx, int ip, int ie);
 
     /**
-     * @brief Model specific implementation of fy
+     * @brief Model-specific implementation of fy
      * @param y model output at current timepoint
      * @param t current time
      * @param x current state
@@ -305,7 +305,7 @@ class AbstractModel {
                     const realtype *w);
 
     /**
-     * @brief Model specific implementation of fdydp
+     * @brief Model-specific implementation of fdydp
      * @param dydp partial derivative of observables y w.r.t. model parameters p
      * @param t current time
      * @param x current state
@@ -321,7 +321,7 @@ class AbstractModel {
                        int ip, const realtype *w, const realtype *dwdp);
 
     /**
-     * @brief Model specific implementation of fdydx
+     * @brief Model-specific implementation of fdydx
      * @param dydx partial derivative of observables y w.r.t. model states x
      * @param t current time
      * @param x current state
@@ -336,7 +336,7 @@ class AbstractModel {
                        const realtype *w, const realtype *dwdx);
 
     /**
-     * @brief Model specific implementation of fz
+     * @brief Model-specific implementation of fz
      * @param z value of event output
      * @param ie event index
      * @param t current time
@@ -349,7 +349,7 @@ class AbstractModel {
                     const realtype *p, const realtype *k, const realtype *h);
 
     /**
-     * @brief Model specific implementation of fsz
+     * @brief Model-specific implementation of fsz
      * @param sz Sensitivity of rz, total derivative
      * @param ie event index
      * @param t current time
@@ -365,7 +365,7 @@ class AbstractModel {
                      const realtype *sx, int ip);
 
     /**
-     * @brief Model specific implementation of frz
+     * @brief Model-specific implementation of frz
      * @param rz value of root function at current timepoint (non-output events
      * not included)
      * @param ie event index
@@ -379,7 +379,7 @@ class AbstractModel {
                      const realtype *p, const realtype *k, const realtype *h);
 
     /**
-     * @brief Model specific implementation of fsrz
+     * @brief Model-specific implementation of fsrz
      * @param srz Sensitivity of rz, total derivative
      * @param ie event index
      * @param t current time
@@ -395,7 +395,7 @@ class AbstractModel {
                       const realtype *h, const realtype *sx, int ip);
 
     /**
-     * @brief Model specific implementation of fdzdp
+     * @brief Model-specific implementation of fdzdp
      * @param dzdp partial derivative of event-resolved output z w.r.t. model
      * parameters p
      * @param ie event index
@@ -411,7 +411,7 @@ class AbstractModel {
                        const realtype *h, int ip);
 
     /**
-     * @brief Model specific implementation of fdzdx
+     * @brief Model-specific implementation of fdzdx
      * @param dzdx partial derivative of event-resolved output z w.r.t. model
      * states x
      * @param ie event index
@@ -426,7 +426,7 @@ class AbstractModel {
                        const realtype *h);
 
     /**
-     * @brief Model specific implementation of fdrzdp
+     * @brief Model-specific implementation of fdrzdp
      * @param drzdp partial derivative of root output rz w.r.t. model parameters
      * p
      * @param ie event index
@@ -442,7 +442,7 @@ class AbstractModel {
                         const realtype *h, int ip);
 
     /**
-     * @brief Model specific implementation of fdrzdx
+     * @brief Model-specific implementation of fdrzdx
      * @param drzdx partial derivative of root output rz w.r.t. model states x
      * @param ie event index
      * @param t current time
@@ -456,7 +456,7 @@ class AbstractModel {
                         const realtype *h);
 
     /**
-     * @brief Model specific implementation of fdeltax
+     * @brief Model-specific implementation of fdeltax
      * @param deltax state update
      * @param t current time
      * @param x current state
@@ -473,7 +473,7 @@ class AbstractModel {
                          const realtype *xdot_old);
 
     /**
-     * @brief Model specific implementation of fdeltasx
+     * @brief Model-specific implementation of fdeltasx
      * @param deltasx sensitivity update
      * @param t current time
      * @param x current state
@@ -496,7 +496,7 @@ class AbstractModel {
                           const realtype *sx, const realtype *stau);
 
     /**
-     * @brief Model specific implementation of fdeltaxB
+     * @brief Model-specific implementation of fdeltaxB
      * @param deltaxB adjoint state update
      * @param t current time
      * @param x current state
@@ -515,7 +515,7 @@ class AbstractModel {
                           const realtype *xB);
 
     /**
-     * @brief Model specific implementation of fdeltaqB
+     * @brief Model-specific implementation of fdeltaqB
      * @param deltaqB sensitivity update
      * @param t current time
      * @param x current state
@@ -535,7 +535,7 @@ class AbstractModel {
                           const realtype *xB);
 
     /**
-     * @brief Model specific implementation of fsigmay
+     * @brief Model-specific implementation of fsigmay
      * @param sigmay standard deviation of measurements
      * @param t current time
      * @param p parameter vector
@@ -545,7 +545,7 @@ class AbstractModel {
                          const realtype *k);
 
     /**
-     * @brief Model specific implementation of fsigmay
+     * @brief Model-specific implementation of fsigmay
      * @param dsigmaydp partial derivative of standard deviation of measurements
      * @param t current time
      * @param p parameter vector
@@ -556,7 +556,7 @@ class AbstractModel {
                             const realtype *p, const realtype *k, int ip);
 
     /**
-     * @brief Model specific implementation of fsigmaz
+     * @brief Model-specific implementation of fsigmaz
      * @param sigmaz standard deviation of event measurements
      * @param t current time
      * @param p parameter vector
@@ -566,7 +566,7 @@ class AbstractModel {
                          const realtype *k);
 
     /**
-     * @brief Model specific implementation of fsigmaz
+     * @brief Model-specific implementation of fsigmaz
      * @param dsigmazdp partial derivative of standard deviation of event
      * measurements
      * @param t current time
@@ -578,7 +578,7 @@ class AbstractModel {
                             const realtype *p, const realtype *k, int ip);
 
     /**
-     * @brief Model specific implementation of fJy
+     * @brief Model-specific implementation of fJy
      * @param nllh negative log-likelihood for measurements y
      * @param iy output index
      * @param p parameter vector
@@ -592,7 +592,7 @@ class AbstractModel {
                      const realtype *sigmay, const realtype *my);
 
     /**
-     * @brief Model specific implementation of fJz
+     * @brief Model-specific implementation of fJz
      * @param nllh negative log-likelihood for event measurements z
      * @param iz event output index
      * @param p parameter vector
@@ -606,7 +606,7 @@ class AbstractModel {
                      const realtype *sigmaz, const realtype *mz);
 
     /**
-     * @brief Model specific implementation of fJrz
+     * @brief Model-specific implementation of fJrz
      * @param nllh regularization for event measurements z
      * @param iz event output index
      * @param p parameter vector
@@ -619,7 +619,7 @@ class AbstractModel {
                       const realtype *sigmaz);
 
     /**
-     * @brief Model specific implementation of fdJydy
+     * @brief Model-specific implementation of fdJydy
      * @param dJydy partial derivative of time-resolved measurement negative
      * log-likelihood Jy
      * @param iy output index
@@ -648,7 +648,7 @@ class AbstractModel {
     virtual void fdJydy_rowvals(SUNMatrixWrapper &dJydy, int index);
 
     /**
-     * @brief Model specific implementation of fdJydsigma
+     * @brief Model-specific implementation of fdJydsigma
      * @param dJydsigma Sensitivity of time-resolved measurement negative
      * log-likelihood Jy w.r.t. standard deviation sigmay
      * @param iy output index
@@ -663,7 +663,7 @@ class AbstractModel {
                             const realtype *sigmay, const realtype *my);
 
     /**
-     * @brief Model specific implementation of fdJzdz
+     * @brief Model-specific implementation of fdJzdz
      * @param dJzdz partial derivative of event measurement negative
      * log-likelihood Jz
      * @param iz event output index
@@ -678,7 +678,7 @@ class AbstractModel {
                         const realtype *sigmaz, const realtype *mz);
 
     /**
-     * @brief Model specific implementation of fdJzdsigma
+     * @brief Model-specific implementation of fdJzdsigma
      * @param dJzdsigma Sensitivity of event measurement negative log-likelihood
      * Jz w.r.t. standard deviation sigmaz
      * @param iz event output index
@@ -693,7 +693,7 @@ class AbstractModel {
                             const realtype *sigmaz, const realtype *mz);
 
     /**
-     * @brief Model specific implementation of fdJrzdz
+     * @brief Model-specific implementation of fdJrzdz
      * @param dJrzdz partial derivative of event penalization Jrz
      * @param iz event output index
      * @param p parameter vector
@@ -706,7 +706,7 @@ class AbstractModel {
                          const realtype *sigmaz);
 
     /**
-     * @brief Model specific implementation of fdJrzdsigma
+     * @brief Model-specific implementation of fdJrzdsigma
      * @param dJrzdsigma Sensitivity of event penalization Jrz w.r.t. standard
      * deviation sigmaz
      * @param iz event output index
@@ -720,7 +720,7 @@ class AbstractModel {
                              const realtype *sigmaz);
 
     /**
-     * @brief Model specific implementation of fw
+     * @brief Model-specific implementation of fw
      * @param w Recurring terms in xdot
      * @param t timepoint
      * @param x vector with the states
@@ -734,7 +734,7 @@ class AbstractModel {
                     const realtype *tcl);
 
     /**
-     * @brief Model specific sparse implementation of dwdp
+     * @brief Model-specific sparse implementation of dwdp
      * @param dwdp Recurring terms in xdot, parameter derivative
      * @param t timepoint
      * @param x vector with the states
@@ -751,19 +751,19 @@ class AbstractModel {
                        const realtype *stcl);
 
     /**
-     * @brief Model specific implementation for dwdp, column pointers
+     * @brief Model-specific implementation for dwdp, column pointers
      * @param dwdp sparse matrix to which colptrs will be written
      */
     virtual void fdwdp_colptrs(SUNMatrixWrapper &dwdp);
 
     /**
-     * @brief Model specific implementation for dwdp, row values
+     * @brief Model-specific implementation for dwdp, row values
      * @param dwdp sparse matrix to which rowvals will be written
      */
     virtual void fdwdp_rowvals(SUNMatrixWrapper &dwdp);
 
     /**
-     * @brief Model specific sensitivity implementation of dwdp
+     * @brief Model-specific sensitivity implementation of dwdp
      * @param dwdp Recurring terms in xdot, parameter derivative
      * @param t timepoint
      * @param x vector with the states
@@ -781,7 +781,7 @@ class AbstractModel {
                        const realtype *stcl, int ip);
 
     /**
-     * @brief Model specific implementation of dwdx, data part
+     * @brief Model-specific implementation of dwdx, data part
      * @param dwdx Recurring terms in xdot, state derivative
      * @param t timepoint
      * @param x vector with the states
@@ -796,19 +796,19 @@ class AbstractModel {
                        const realtype *w, const realtype *tcl);
 
     /**
-     * @brief Model specific implementation for dwdx, column pointers
+     * @brief Model-specific implementation for dwdx, column pointers
      * @param dwdx sparse matrix to which colptrs will be written
      */
     virtual void fdwdx_colptrs(SUNMatrixWrapper &dwdx);
 
     /**
-     * @brief Model specific implementation for dwdx, row values
+     * @brief Model-specific implementation for dwdx, row values
      * @param dwdx sparse matrix to which rowvals will be written
      */
     virtual void fdwdx_rowvals(SUNMatrixWrapper &dwdx);
 
     /**
-     * @brief Model specific implementation of fdwdw, no w chainrule (Py)
+     * @brief Model-specific implementation of fdwdw, no w chainrule (Py)
      * @param dwdw partial derivative w wrt w
      * @param t timepoint
      * @param x Vector with the states
@@ -823,13 +823,13 @@ class AbstractModel {
                        const realtype *w, const realtype *tcl);
 
     /**
-     * @brief Model specific implementation of fdwdw, colptrs part
+     * @brief Model-specific implementation of fdwdw, colptrs part
      * @param dwdw sparse matrix to which colptrs will be written
      */
     virtual void fdwdw_colptrs(SUNMatrixWrapper &dwdw);
 
     /**
-     * @brief Model specific implementation of fdwdw, rowvals part
+     * @brief Model-specific implementation of fdwdw, rowvals part
      * @param dwdw sparse matrix to which rowvals will be written
      */
     virtual void fdwdw_rowvals(SUNMatrixWrapper &dwdw);
